/*
 * PmodTC1.c
 *
 *  Created on: Dec 16, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"
#include "PmodTC1.h"

SPI_HandleTypeDef pmodTc1Spi;

void PmodTC1_Config(void)
{
  // Configure the SPI connected to the Pmod module. Only the MISO line is required.
  pmodTc1Spi.Instance = SPI1;
  pmodTc1Spi.Init.Mode = SPI_MODE_MASTER;
  pmodTc1Spi.Init.Direction = SPI_DIRECTION_2LINES_RXONLY;
  pmodTc1Spi.Init.DataSize = SPI_DATASIZE_16BIT;
  pmodTc1Spi.Init.CLKPolarity = SPI_POLARITY_LOW;
  pmodTc1Spi.Init.CLKPhase = SPI_PHASE_2EDGE;
  pmodTc1Spi.Init.NSS = SPI_NSS_SOFT;
  pmodTc1Spi.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64;
  pmodTc1Spi.Init.FirstBit = SPI_FIRSTBIT_MSB;
  pmodTc1Spi.Init.TIMode = SPI_TIMODE_DISABLE;
  pmodTc1Spi.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  pmodTc1Spi.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;

  HAL_SPI_Init(&pmodTc1Spi);
}

void HAL_SPI_MspInit(SPI_HandleTypeDef *hspi)
{
  // Initialize GPIO used by the SPI1 peripheral. The CS is controlled by the software (PB0 pin).
  __HAL_RCC_SPI1_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI1;
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}

void PmodTC1_GetValues(float* temp, float* tempRef, uint8_t* status)
{
  uint16_t data[2] = {0};

  // Read all values coded on 32-bit SPI output.
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_Receive(&pmodTc1Spi, (uint8_t*)data, 2, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  // Check the temperature sign.
  uint16_t tempSign = data[0] & 0x8000;
  data[0] &= ~0x8000;

  // Convert the temperature to Celsius degrees and apply sign.
  *temp = (data[0] >> 2) / 4.0;
  if(tempSign != 0)
    *temp *= -1;

  // Check the reference temperature sign.
  uint16_t tempRefSign = data[1] & 0x8000;
  data[1] &= ~0x8000;

  // Convert the reference temperature to Celsius degrees and apply sign.
  *tempRef = (data[1] >> 4) / 16.0;
  if(tempRefSign != 0)
    *tempRef *= -1;

  // Read the status bits.
  *status = data[1] & 0x03;
}
